@@ -1,5 +1,6 @@
 use std::{
     borrow::Cow,
+    convert::TryInto,
     ffi::{CStr, CString, OsStr, OsString},
     os::unix::prelude::OsStrExt,
     path::{Path, PathBuf},
@@ -31,7 +32,6 @@ impl UnixString {
         Self::default()
     }
 
-    #[inline(always)]
     fn extend_slice(&mut self, slice: &[u8]) {
         let removed = self.inner.remove(self.inner.len() - 1);
         debug_assert!(removed == 0);
@@ -180,7 +180,6 @@ impl UnixString {
         self.as_c_str().as_ptr()
     }
 
-    #[inline(always)]
     fn inner_without_nul_terminator(&self) -> &[u8] {
         &self.inner[0..self.inner.len() - 1]
     }
@@ -238,6 +237,41 @@ impl UnixString {
         Ok(std::str::from_utf8(self.inner_without_nul_terminator())?)
     }
 
+    /// Extends a `UnixString` by copying from a raw C string
+    ///
+    /// # Safety
+    ///
+    /// This method is unsafe for a number of reasons:
+    /// * The total size of the raw C string must be smaller than `isize::MAX` bytes in memory.
+    /// * There is no guarantee to the validity of `ptr`.
+    /// * There is no guarantee that the memory pointed to by `ptr` contains a
+    ///   valid nul terminator byte at the end of the string.
+    /// * It is not guaranteed that the memory pointed by `ptr` won't change
+    ///   while this operation occurs.
+    ///
+    /// This function uses [`CStr::from_ptr`](std::ffi::CStr::from_ptr) internally, so check it out for more information.
+    ///
+    pub unsafe fn extend_from_ptr(&mut self, ptr: *const libc::c_char) -> Result<()> {
+        let cstr = CStr::from_ptr(ptr);
+        let bytes = cstr.to_bytes();
+        self.push_bytes(bytes)
+    }
+
+    /// This function will check if the inner bytes of this `UnixString` (without its null terminator) consists of valid UTF-8, and if so, returns a `String` reusing the `UnixString`'s buffer.
+    ///
+    /// If the inner bytes contain invalid UTF-8, then a new `String` will be allocated with the invalid bytes replaced with the Unicode replacement codepoint.
+    pub fn into_string_lossy(self) -> String {
+        let bytes = self.into_bytes();
+        match String::from_utf8_lossy(&bytes) {
+            Cow::Owned(new_string) => new_string,
+            Cow::Borrowed(_) => {
+                // Safety: String::from_utf8_lossy returns a reference when the supplied input
+                // is valid UTF-8. Constructing a String unchecked, therefore, is safe.
+                unsafe { String::from_utf8_unchecked(bytes) }
+            }
+        }
+    }
+
     /// Converts a `UnixString` into a String if the bytes of the `UnixString` are valid UTF-8.
     ///
     /// If you are sure that the byte slice is valid UTF-8 and you don’t want to incur the overhead of the validity check, there is an unsafe version of this function, [`UnixString::into_string_unchecked`](UnixString::into_string_unchecked), which has the same behavior but skips the check.
@@ -278,13 +312,11 @@ impl UnixString {
         String::from_utf8_unchecked(self.into_bytes())
     }
 
-    /// Converts this `UnixString` into a String in a lossy manner.
+    /// This function will check if the inner bytes of this `UnixString` (without its null terminator) consists of valid UTF-8, and if so, returns a string slice without copying.
     ///
-    /// If the inner bytes invalid UTF-8, then the invalid bytes are replaced with the Unicode replacement codepoint.
-    ///  
-    /// If the bytes in this `UnixString` are valid UTF-8, no copying is done.
+    /// If the inner bytes contain invalid UTF-8, then a new `String` will be allocated with the invalid bytes replaced with the Unicode replacement codepoint.
     pub fn to_string_lossy(&self) -> Cow<str> {
-        self.as_os_str().to_string_lossy()
+        self.as_c_str().to_string_lossy()
     }
 
     /// Gets the underlying byte view of this `UnixString` *without* the nul terminator.
@@ -357,6 +389,45 @@ impl UnixString {
         bytes.remove(bytes.len() - 1);
         bytes
     }
+
+    /// Converts a `CString` into an `UnixString`.
+    ///
+    /// This operation is zero-cost and does not fail.
+    ///
+    /// This operation fails if the `CString` has any interior zero byte but a zero byte at the last position is acceptable.
+    pub fn from_cstring(cstring: CString) -> Self {
+        cstring.into()
+    }
+
+    /// Converts a `PathBuf` into an `UnixString`.
+    ///
+    /// A validation will be done to ensure that the `PathBuf` has no interior zero bytes.
+    /// Other than that, this operation is zero-cost.
+    ///
+    /// This operation fails if the `PathBuf` has any interior zero byte but a zero byte at the last position is acceptable.
+    pub fn from_pathbuf(pathbuf: PathBuf) -> Result<Self> {
+        pathbuf.try_into()
+    }
+
+    /// Converts a `String` into an `UnixString`.
+    ///
+    /// A validation will be done to ensure that the `String` has no interior zero bytes.
+    /// Other than that, this operation is zero-cost.
+    ///
+    /// This operation fails if the `String` has any interior zero byte but a zero byte at the last position is acceptable.
+    pub fn from_string(string: String) -> Result<Self> {
+        string.try_into()
+    }
+
+    /// Converts an `OsString` into an `UnixString`.
+    ///
+    /// A validation will be done to ensure that the `OsString` has no interior zero bytes.
+    /// Other than that, this operation is zero-cost.
+    ///
+    /// This operation fails if the `OsString` has any interior zero byte but a zero byte at the last position is acceptable.
+    pub fn from_os_string(os_string: OsString) -> Result<Self> {
+        os_string.try_into()
+    }
 }
 
 impl From<CString> for UnixString {
